<?php
add_action('the_content','rpur_content_filter');
add_action('the_title','rpur_title_filter');
add_filter('wp_head', 'rpur_init');
$rpur_options = get_option('rpur_options');

function rpur_content_filter($content){
	if(is_admin())
		return $content;
	
	$id = get_post_meta(get_the_ID(), 'rpur_feed_id', true);
	
	if(!$id)
		return $content;//might be normal content	

	$feed = rpur_get_feed($id);

	global $rpur_options;
	if($rpur_options['word_limit'] >0 )
		$content = rpur_truncate($content, $rpur_options['word_limit']);

	if(!$feed){

		$content=rpur_rewrite($content);

	}elseif($feed['rewrite']==='Yes'){
		
		$content=rpur_rewrite($content);

	}
	
	if($rpur_options['display_article_source']){

		$sourcelink=get_post_meta(get_the_ID(), 'rpur_sourcepermalink', true);

		if($sourcelink)
			$link='<p>'.rpur_custom_template($sourcelink,$id).'</p>';

		$content=$content.$link;
	}

	

	return $content;


}
function rpur_title_filter($title){
	if(is_admin())
		return $title;
	
	$id = get_post_meta(get_the_ID(), 'rpur_feed_id', true);

	if(!$id)
		return $title;//might be normal content
	
	$feed = rpur_get_feed($id);
	if(!$feed){

		$title=rpur_rewrite($title);

	}elseif($feed['rewrite']==='Yes'){
		$title=rpur_rewrite($title);
	}
	
		
	return $title;

}

function rpur_filter_post($title){
	global $rpur_options;
	
	$include = $rpur_options['include'];
	$exclude = $rpur_options['exclude'];

	
	if($include){
		$includeArr=explode(',',$include);
		foreach($includeArr as $i){
		
			if( 1 === preg_match('@\b'.$i.'\b@',$title) ){
				rpur_log('Fetch <b>'.$title.'</b> matching <b>'.$i.'</b>.');
				return false;
			}
		
		}
		rpur_log('No posts titles found matching <b>'.$include.'</b>.');
		exit;
	}

	if($exclude){
		$excludeArr=explode(',',$exclude);
		foreach($excludeArr as $e){
			if( 1 === preg_match('@\b'.$e.'\b@',$title) ){
				rpur_log('Skip <b>'.$title.'</b> matching <b>'.$e.'</b>.');
				return true;
			}
		
		}
		
	}
	
	return false;
		
}




function rpur_custom_template($sourcelink,$feed_id){

	global $rpur_options;
	$custom_template = $rpur_options['custom_template'];

	if( false ===  strpos($custom_template, '%SOURCE_URL%' ) )
		$custom_template .= ' <a href="%SOURCE_URL%">%SOURCE_URL%</a> ';

	
	
	$feed = rpur_get_feed($feed_id);
	
	$user = get_userdata($feed['author']);
	$author = $user->nickname;

	$category = get_cat_name($feed['category']);

	$feed_url = $feed['url'];

	$feed_name = $feed['name'];
	
	$custom_template = str_replace(
		array('%SOURCE_URL%','%AUTHOR%','%CATEGORY%','%FEED_URL%','%FEED_NAME%'),
		array($sourcelink,$author,$category,$feed_url,$feed_name),
		$custom_template

	);
	
	return $custom_template;

}
function rpur_rewrite($text,$keyword=''){
	
	global $syndb;
	
	
	if(!$syndb) 
		return $text;
	
	global $rpur_options;

	$keepxx=$rpur_options['dont_rewrite'];

	if($keepxx){
		
		$keepxx=preg_split('(\r\n|\n|\r)',$keepxx);

		foreach ($keepxx as $k => $v){
		
			$text=str_replace(' '.$v.' '," keepxx[$k] ",$text);
		}
	}
	

	$synlist=$text;
	$synchar = array("(", ")", "[", "]", "?", ".", ",", "|", "\$", "*", "+", "^", "{", "}");
	$synlist=str_replace($synchar," ",$synlist);
	$synlist=str_replace("  "," ",$synlist);
	$my_synlist = explode(" ",$synlist);

	if (count($my_synlist)>0){

		for($i=0;$i<sizeof($my_synlist);$i++){

			$syn_replace=$my_synlist[$i];
			$syn_replace=str_replace(" ","",$syn_replace);
			$syn_replace=" ".$syn_replace." ";
			$syn_replace_target=$syndb["$syn_replace"];
			
			if( ($syn_replace!="") && ($syn_replace_target!="") ){
				$text = str_replace($syn_replace,$syn_replace_target,$text);
			}
		}
	}
	
	if($keepxx){
		
		foreach ($keepxx as $k => $v){
		
			$text=str_replace(" keepxx[$k] ",' '.$keepxx[$k].' ',$text);
		}
	}

	return $text;

}
function rpur_run_cron(){
	
	global $rpur_options;
	if($rpur_options['enable_cron_job'] == true ){

		rpur_log('<b>Processing feeds from Cron URL</b>');
		rpur_process_feeds();

	}else{
		rpur_log('<b>Please enable cron job in the first place.</b>');
	}	
		
	


}
function rpur_init(){
	
	global $rpur_options;
	
	if(false == $rpur_options['enable_cron_job']){
		
		$now_time=time();

		$interval=$rpur_options['frequency']*60*60;
		$lastactive=$rpur_options['lastactive'];
		if(!$lastactive){
			$lastactive=$now_time;
			$rpur_options['lastactive']=$lastactive;
			update_option('rpur_options', $rpur_options);
		
		}
	
		if ( ($now_time - $lastactive ) >= $interval ) {
		
			$rpur_options['lastactive'] = $now_time+$interval;
			update_option('rpur_options', $rpur_options);
			rpur_process_feeds();
		}
	}
	
}



function rpur_process_feeds(){

	@set_time_limit(0);
	
	
	$feeds=rpur_get_feeds();
	if($feeds){

		rpur_log("<b>----------Processing all feeds-------------</b>");

		foreach($feeds as $feed){
		
			rpur_process_feed($feed);

		}
	}else{
		rpur_log("No feeds data found!");
	}
	

}

function rpur_truncate($text,$count) { 
	$count=intval($count);
	if(  $count<1 || substr_count($text,' ') < $count )
		return $text;

	$temp = explode(' ', $text);

	$text = implode(' ', array_slice($temp, 0, $count));

	return $text;
	
}

function rpur_process_feed($feed){
	@set_time_limit(0);
	if(empty($feed['url'])){
		
		return false;
	}
	
	rpur_log("Processing feed <b>{$feed['url']}</b>");

	$lastactive = $feed['lastactive'];
	
	$now=time();

	$frequency = $feed['frequency']*60;
	
	if( ($now - $lastactive) >= $frequency ){
		
		rpur_update_feed_lastactive($feed['id'],$now);
		
	}else{
		
		rpur_log("It's not the time to update <b>{$feed['url']}</b>. <a href='http://www.wprssposter.com/faq.html#cronjob'>Explanation</a>");
		return false;
	}

	global $rpur_options;

	$max_items=$feed['max_items'];

	if(empty($max_items)||!is_numeric($max_items))

		$max_items=3;
	
	$simplepie = rpur_fetch_feed($feed['url'],$max_items);
	
	$error=$simplepie->error();

	if($error){
		
		rpur_log("Feed Error: <b>$error</b>");
		return false;
	}
	
    	$count = 0;
    	
	foreach($simplepie->get_items() as $item){
		
		if( rpur_filter_post( $item->get_title() ) )
			continue;
	
		if( rpur_is_duplicate(  $item->get_title() ) ){
       			rpur_log('Filtering duplicate post');
        		continue;
		}
		
		if(false == rpur_process_item($item,$feed,$count))
			continue;

		$count++;
		if($count == $max_items)
			break;
      	}

    	if($count==0)
		rpur_log("No new or qualified post for <b>{$feed['url']}</b> <a href='http://www.wprssposter.com/faq.html#newandqualified'>Explanation</a>");
	else
		rpur_log( "Fetch $count posts from <b>{$feed['url']}</b>" );
		
    	
    	return true;
    

}

function rpur_process_item($item,$feed,$count=0){
	

	global $wpdb;

	$title = $item->get_title();

	$link = $item->get_permalink();

	if(false !== strpos($link,'news.google.com')){
		$link=urldecode(substr($link,strpos($link,'url=')+4));
	}elseif(false !== strpos($link,'/**')){
		$link=urldecode(substr($link,strpos($link,'/**')+3));
		
	}	
	
	$content = rpur_full_feed($link);
	
	if(!$content){
		//rpur_log("Cannot grab full content from <b>$link</b>");
		return false;
	}
	$title=rpur_title_fix($title);

	$content=rpur_content_fix($content);
	
	if(empty($title)||empty($content)){
		
		return false;
	}
	global $rpur_options;
	if( $rpur_options['cache_image'] == true ){
		
		$content=rpur_parse_images($content,$item->get_base());
	}
	$meta = array(
	     'rpur_feed_id' => $feed['id'],
	     'rpur_sourcepermalink' => $link
    	);  


	$rpur_date = time() + ( $count * $feed['post_interval'] * 60 * 60 );
	
	
	rpur_insert_post($title,$content,$feed['category'],$feed['tags'],$meta,$feed['post_status'],$rpur_date,$feed['author']);
 	rpur_log("<b>{$title}</b> added");
	global $wpdb;
	global $rpur_options;
	$hash=rpur_item_hash($item->get_title());
	$sql="insert into {$rpur_options['db']['post']} (hash) values ('{$hash}')";
	$wpdb->query($sql);
	
	return true;


}
function rpur_title_fix($title){
	if($title && strpos($title,' - ')){
		
		$backup=$title;
		$backup=preg_replace('/([-])/','$1[D]',$backup);

		$backup=explode('[D]',$backup);

		if( strlen($backup[0])>10 || count($backup)>=2 )
			unset($backup[count($backup)-1]);
		else
			return $title;
		
		$title=trim(implode('',$backup),' - ');
	}
	return $title;

}

function rpur_content_fix($text){

	global $rpur_options;
	
	$remove_pairs=preg_split('(\r\n|\r|\n)',$rpur_options['remove_pairs']);
	if($remove_pairs){

		foreach($remove_pairs as $pair){

			$text=@preg_replace('@'.$pair.'@Ums','',$text);
		}		

	}


	preg_match_all('@(<a.+?href=\".+?\">)(.*?</a>)@',$text,$m);
	$urls = $m[1];
	if($urls){
		foreach($urls as $pos => $link){
			if(false === stripos($link,'http://') && false === stripos($link,'https://')){
		
				$text=str_replace($link,'',$text);
		
				$text=str_replace($m[2][$pos],str_replace('</a>','',$m[2][$pos]),$text);
			}
		}
	}
	
	$text=preg_replace("/[&|#|&#]+[a-z0-9]+;/i","",$text);
	$text=preg_replace('@<[dtrx][^>]*>@','',$text);
	$text=preg_replace('@</[dtrx][^>]*>@','',$text);
	return $text;

}
function rpur_parse_images($content,$link){
	preg_match_all('/<img(.+?)src=\"(.+?)\"(.*?)>/', $content, $images);
	$urls = $images[2];
        
      	if(count($urls)){
		
		foreach($urls as $pos => $url){
			$oldurl=$url;
			$meta=parse_url($url);
			
			if(!isset($meta['host'])){

				$meta=parse_url($link);
				$url=$meta['scheme'].'://'.$meta['host'].'/'.$url;
				
   			}
			
          		$newurl = rpur_cache_image($url);
          		if($newurl)
            			$content = str_replace($oldurl, $newurl, $content);
			else
				$content = str_replace($images[0][$pos],'',$content);
        	} 
        }
	return $content;
   	


}
function rpur_cache_image($url){
	if( strpos($url, "icon_") !== FALSE)
	      return false;
	global $rpur_options;
	$contents = rpur_get_file($url);

	if( !$contents )
		return false;

	$basename = basename($url);
	$paresed_url = parse_url($basename);
	
	$filename = substr(md5(time()), 0, 5) . '_' . $paresed_url['path'];
	
	$cachepath = RPUR_CACHE;
	$pluginpath = RPUR_URL_ROOT;
    	$real_cachepath=dirname(__FILE__).'/'.$cachepath;
	if(is_writable(	$real_cachepath ) ){
		
		if($contents){

			file_put_contents($real_cachepath . $filename, $contents);
			$i=@exif_imagetype($real_cachepath . $filename);
			if($i)
				return $pluginpath . $cachepath . rawurlencode($filename);
		}
	}else{
		
		rpur_log($real_cachepath . " directory is not writable" );
		
	}
    
	return false;

}
function rpur_insert_post($title,$content,$category=array(1),$tags_input='',$meta='',$post_status='publish',$rpur_date,$post_author=1){
	
	$category=(array)$category;
	
	$postid = wp_insert_post(array(
    		'post_title' 	          => $title,
  		'post_content'  	  => $content,
  		'post_category'           => $category,
		'tags_input'		  => $tags_input,
		'post_status'		   => $post_status,
		'post_author'             => $post_author,
  		"post_date" => get_date_from_gmt(date("Y-m-d H:i:s", $rpur_date)),
		
		
    	));
	
	
	if($meta)
		foreach($meta as $key => $value) 
			rpur_insert_post_meta($postid, $key, $value);
	
		
	return $postid;

}

function rpur_full_feed($permalink){
	@require_once(RPURINC.'readability.php');

	if ($permalink && $html = rpur_get_file($permalink)) {
		
		$html = rpur_convert_to_utf8($html);

		$content = grabArticleHtml($html);

		
	}else
		return false;
	
	if( false !== stripos($content,'readability was unable to parse this page for content') )
		return false;
	if( false !== stripos($content, 'return go_back();') )
		return false;
	
	return $content;

}
function rpur_insert_post_meta($postid, $key, $value) {
	global $wpdb;
		
	$result = $wpdb->query( "INSERT INTO $wpdb->postmeta (post_id,meta_key,meta_value ) " 
					                . " VALUES ('$postid','$key','$value') ");
					
	return $wpdb->insert_id;		
}

function rpur_is_duplicate($title){

	global $wpdb;
	global $rpur_options;

	$hash = rpur_item_hash($title);

	$row = $wpdb->get_row("SELECT * FROM {$rpur_options['db']['post']} "
                          . "WHERE hash = '{$hash}'");  
	if($row)
		return true;
	return false;


}

function rpur_item_hash($data){

	return sha1($data);
}
function rpur_get_file($url){

	if(ini_get('allow_url_fopen') != 1) {
		@ini_set('allow_url_fopen', '1');
	}

	if(ini_get('allow_url_fopen') != 1) {
		
		$ch = curl_init();
 
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); //Set curl to return the data instead of printing it to the browser.
		curl_setopt($ch, CURLOPT_URL, $url);
 
		$data = curl_exec($ch);
		curl_close($ch);
 
		return $data;

    
	} else {
		return @file_get_contents($url);
	}
	
	return false;

 
}
function rpur_fetch_feed($url,$max_items){
    if(!class_exists('SimplePie'))
	require_once(RPURINC.'simplepie.class.php');
    $url=str_replace(' ','+',$url);

    $feed = new SimplePie();
    $feed->enable_order_by_date(false); 
    $feed->set_feed_url($url);
    $feed->set_item_limit($max_items);
    $feed->set_stupidly_fast(true);
    $feed->enable_cache(false);    
    $feed->init();
    $feed->handle_content_type(); 
    
    return $feed;

}

function rpur_get_feeds(){

	global $wpdb;

	global $rpur_options;

	$sql="select * from {$rpur_options['db']['feed']} order by id desc";

	return $wpdb->get_results($sql,ARRAY_A);


}

function rpur_get_feed($id){

	global $wpdb;

	global $rpur_options;

	$sql="select * from {$rpur_options['db']['feed']} where id={$id}";
	$rtn=$wpdb->get_results($sql,ARRAY_A);
	if(is_array($rtn[0]))
		return $rtn[0];
	return $rtn;


}
function rpur_update_feed_lastactive($id,$lastactive){
	global $wpdb;
	global $rpur_options;
	$sql = "update {$rpur_options['db']['feed']} set lastactive='{$lastactive}' where id={$id}";
	
	$result=$wpdb->query($sql);
	
	return $result;	

}
function rpur_delete_feed($data){

	global $wpdb;

	global $rpur_options;

	$sql= "delete from {$rpur_options['db']['feed']} where id={$data['id']} ";

	$result=$wpdb->query($sql);

	
	return $result;	

}

function rpur_update_feed($data){

	global $wpdb;

	global $rpur_options;

$sql = "update {$rpur_options['db']['feed']} set name='{$data['name']}',url='{$data['url']}',tags='{$data['tags']}',category='{$data['category']}',frequency='{$data['frequency']}',max_items='{$data['max_items']}', post_status='{$data['post_status']}', rewrite='{$data['rewrite']}', post_interval ='{$data['post_interval']}', author='{$data['author']}' where id={$data['id']}";
	
	$result=$wpdb->query($sql);
	
	return $result;	
	
}
function rpur_insert_feed($data){
		
	global $wpdb;

	global $rpur_options;

	$sql = "INSERT INTO {$rpur_options['db']['feed']} (name,url,tags,category,frequency,max_items,rewrite,post_status,post_interval,author) VALUES ('{$data['name']}','{$data['url']}','{$data['tags']}','{$data['category']}','{$data['frequency']}','{$data['max_items']}','{$data['rewrite']}','{$data['post_status']}','{$data['post_interval']}','{$data['author']}')";
	
	$result = $wpdb->query($sql);

	return $result;	


}


function rpur_log($message){
   	global $wpdb;
    	global $rpur_options;
    	$message = $wpdb->escape($message);
      	$time = current_time('mysql', true);
      	$wpdb->query("INSERT INTO {$rpur_options['db']['log']} (message, created_on) VALUES ('{$message}', '{$time}') "); 
    		
}
function rpur_get_logs($page,$logs_per_page){
	global $wpdb;
  	global $rpur_options;
	

	if($page == 0) $page = 1;
	$page--;
	$orderby = "created_on";
	$ordertype = 'ASC';
	$start = $page * $logs_per_page;
	$end = $start + $logs_per_page;
	$limit = "LIMIT {$start}, {$end}";
	
  	
  	return $wpdb->get_results("SELECT * FROM {$rpur_options['db']['log']} ORDER BY $orderby $ordertype $limit");
}
function rpur_timezone_mysql($format, $time){
    return mysql2date($format, $time);    
}
//////////////////////////////////////////////
// Convert $html to UTF8
// (uses HTTP headers and HTML to find encoding)
// adapted from http://stackoverflow.com/questions/910793/php-detect-encoding-and-make-everything-utf-8
//////////////////////////////////////////////
function rpur_convert_to_utf8($html, $header=null)
{
	$accept = array(
		'type' => array('application/rss+xml', 'application/xml', 'application/rdf+xml', 'text/xml', 'text/html'),
		'charset' => array_diff(mb_list_encodings(), array('pass', 'auto', 'wchar', 'byte2be', 'byte2le', 'byte4be', 'byte4le', 'BASE64', 'UUENCODE', 'HTML-ENTITIES', 'Quoted-Printable', '7bit', '8bit'))
	);
	$encoding = null;
	if ($html || $header) {
		if (is_array($header)) $header = implode("\n", $header);
		if (!$header || !preg_match_all('/^Content-Type:\s+([^;]+)(?:;\s*charset=([^;"\'\n]*))?/im', $header, $match, PREG_SET_ORDER)) {
			// error parsing the response
		} else {
			$match = end($match); // get last matched element (in case of redirects)
			if (!in_array(strtolower($match[1]), $accept['type'])) {
				// type not accepted
				// TODO: avoid conversion
			}
			if (isset($match[2])) $encoding = trim($match[2], '"\'');
		}
		if (!$encoding) {
			if (preg_match('/^<\?xml\s+version=(?:"[^"]*"|\'[^\']*\')\s+encoding=("[^"]*"|\'[^\']*\')/s', $html, $match)) {
				$encoding = trim($match[1], '"\'');
			} elseif(preg_match('/<meta\s+http-equiv=["\']Content-Type["\'] content=["\'][^;]+;\s*charset=([^;"\'>]+)/i', $html, $match)) {
				if (isset($match[1])) $encoding = trim($match[1]);
			}
		}
		
		if (!$encoding) {
			$encoding = 'utf-8';
		} else {
			if (!in_array($encoding, array_map('strtolower', $accept['charset']))) {
				// encoding not accepted
				// TODO: avoid conversion
			}
			if (strtolower($encoding) != 'utf-8') {
				if (strtolower($encoding) == 'iso-8859-1') {
					// replace MS Word smart qutoes
					$trans = array();
					$trans[chr(130)] = '&sbquo;';    // Single Low-9 Quotation Mark
					$trans[chr(131)] = '&fnof;';    // Latin Small Letter F With Hook
					$trans[chr(132)] = '&bdquo;';    // Double Low-9 Quotation Mark
					$trans[chr(133)] = '&hellip;';    // Horizontal Ellipsis
					$trans[chr(134)] = '&dagger;';    // Dagger
					$trans[chr(135)] = '&Dagger;';    // Double Dagger
					$trans[chr(136)] = '&circ;';    // Modifier Letter Circumflex Accent
					$trans[chr(137)] = '&permil;';    // Per Mille Sign
					$trans[chr(138)] = '&Scaron;';    // Latin Capital Letter S With Caron
					$trans[chr(139)] = '&lsaquo;';    // Single Left-Pointing Angle Quotation Mark
					$trans[chr(140)] = '&OElig;';    // Latin Capital Ligature OE
					$trans[chr(145)] = '&lsquo;';    // Left Single Quotation Mark
					$trans[chr(146)] = '&rsquo;';    // Right Single Quotation Mark
					$trans[chr(147)] = '&ldquo;';    // Left Double Quotation Mark
					$trans[chr(148)] = '&rdquo;';    // Right Double Quotation Mark
					$trans[chr(149)] = '&bull;';    // Bullet
					$trans[chr(150)] = '&ndash;';    // En Dash
					$trans[chr(151)] = '&mdash;';    // Em Dash
					$trans[chr(152)] = '&tilde;';    // Small Tilde
					$trans[chr(153)] = '&trade;';    // Trade Mark Sign
					$trans[chr(154)] = '&scaron;';    // Latin Small Letter S With Caron
					$trans[chr(155)] = '&rsaquo;';    // Single Right-Pointing Angle Quotation Mark
					$trans[chr(156)] = '&oelig;';    // Latin Small Ligature OE
					$trans[chr(159)] = '&Yuml;';    // Latin Capital Letter Y With Diaeresis
					$html = strtr($html, $trans);
				}
				if(!class_exists('SimplePie_Misc'))
					require_once(RPURINC.'simplepie.class.php');

				$html = SimplePie_Misc::change_encoding($html, $encoding, 'utf-8');

				/*
				if (function_exists('iconv')) {
					// iconv appears to handle certain character encodings better than mb_convert_encoding
					$html = iconv($encoding, 'utf-8', $html);
				} else {
					$html = mb_convert_encoding($html, 'utf-8', $encoding);
				}
				*/
				
			}
			
		}
	}
	return $html;
}
if ( ! function_exists( 'exif_imagetype' ) ) {
    function exif_imagetype ( $filename ) {
        return @getimagesize( $filename );
    }
}
?>
